package org.jboss.qa.brms.performance.bre;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.impl.ClassPathResource;
import org.drools.runtime.StatefulKnowledgeSession;
import org.jboss.qa.brms.performance.domain.Account;
import org.jboss.qa.brms.performance.domain.Address;
import org.jboss.qa.brms.performance.domain.Customer;
import org.jboss.qa.brms.performance.domain.Transaction;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

public class StandardOperatorsTest {
    private KnowledgeBase kbase;

    @Test
    public void test() {
        createKBase("expert-standard-operators-100.drl");
        // dummy warmup
        System.out.println("Dummy warm-up....");
        for (int i = 0; i < 5; i++) {
            performLoop();
            System.gc();
        }
        System.out.println("Dummy warm-up done.");
        for (int i = 0; i < 10; i++) {
            performLoop();
            System.gc();
        }
    }

    private void performLoop() {
        StatefulKnowledgeSession ksession = createStatefulKSession();
        List<Object> facts = getFacts();
        long start = System.nanoTime();
        for (Object fact : facts) {
            ksession.insert(fact);
        }
        ksession.fireAllRules();
        long took = System.nanoTime() - start;
        System.out.println("Loop took " + took / 1000 / 1000 + "ms");
        ksession.dispose();
    }

    private void createKBase(String drlName) {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(new ClassPathResource(drlName), ResourceType.DRL);
        KnowledgeBaseConfiguration kbconf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
        if (kbuilder.hasErrors()) {
            throw new RuntimeException(kbuilder.getErrors().toString());
        }

        kbase = KnowledgeBaseFactory.newKnowledgeBase(kbconf);
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
    }

    private StatefulKnowledgeSession createStatefulKSession() {
        StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
        return ksession;
    }

    private List<Object> getFacts() {
        List<Object> facts = new ArrayList<Object>();
        for (int i = 0; i < 100000; i++) {
            facts.add(new Account());
            facts.add(new Address());
            facts.add(new Customer("" + i));
            facts.add(new Transaction());

        }
        return facts;
    }
}
